package org.jboss.qa.brms.processpersistence;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.process.DefaultProcessEventListener;
import org.drools.event.rule.DefaultWorkingMemoryEventListener;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.StatefulKnowledgeSession;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class ProcessPersistenceTest {

    public static void main(String[] args) {
        StatefulKnowledgeSession ksession = newSession();

        ksession.addEventListener(new DefaultProcessEventListener());
        ksession.addEventListener(new DefaultWorkingMemoryEventListener());

        ksession.startProcess("ruleflow");

        ksession.fireAllRules();


        System.out.println("All finished...");
    }

    private static StatefulKnowledgeSession newSession() {
        System.setProperty("java.naming.factory.initial", "bitronix.tm.jndi.BitronixInitialContextFactory");

        PoolingDataSource ds = new PoolingDataSource();
        ds.setUniqueName( "jdbc/BitronixJTADataSource" );
        ds.setClassName( "bitronix.tm.resource.jdbc.lrc.LrcXADataSource" );
        ds.setMaxPoolSize( 3 );
        ds.setAllowLocalTransactions(true);
        ds.getDriverProperties().put("driverClassName", "org.hsqldb.jdbcDriver");
        ds.getDriverProperties().put("url", "jdbc:hsqldb:file:/tmp/testDB");
        ds.init();

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("ruleflow.rf", ProcessPersistenceTest.class), ResourceType.DRF);

        if (kbuilder.hasErrors()) {
            throw new RuntimeException(kbuilder.getErrors().toString());
        }

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        KnowledgeSessionConfiguration ksconf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();

        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.drools.persistence.jpa");
        Environment env = KnowledgeBaseFactory.newEnvironment();
        env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        env.set(EnvironmentName.TRANSACTION_MANAGER, TransactionManagerServices.getTransactionManager());

        return JPAKnowledgeService.newStatefulKnowledgeSession(kbase, ksconf, env);
    }
}
